/**
 * Copyright (c) 2011, REMPL.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution.
 * 3) Neither the name of the REMPL.com nor the names of its
 * contributors may be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.rempl.api;

import com.ymock.util.Logger;
import com.ymock.util.RuntimeProblem;

// for file processing
import java.io.File;
import java.io.StringReader;
import java.io.StringWriter;

// XPath supplementary
import javax.xml.namespace.QName;

// JAXP
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

// DOM Source, transformations
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

// XPath API
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathFactory;

// string and file processing
import org.apache.commons.io.FileUtils;

// DOM
import org.w3c.dom.Document;
import org.w3c.dom.Node;

// SAX
import org.xml.sax.InputSource;

/**
 * Parser of incoming XML file (in ROM schema).
 *
 * @author Yegor Bugayenko (yegor@rempl.com)
 * @version $Id: ROMUtils.java 1614 2011-07-10 19:05:43Z guard $
 */
final class ROMUtils {

    /**
     * Document builder factory, DOM.
     */
    private static DocumentBuilderFactory dbFactory =
        DocumentBuilderFactory.newInstance();

    /**
     * XPath factory, DOM.
     */
    private static XPathFactory xpathFactory =
        XPathFactory.newInstance();

    /**
     * DOM transformer factory, DOM.
     */
    private static TransformerFactory transFactory =
        TransformerFactory.newInstance();

    /**
     * Parse the file provided.
     * @param file Absolute <tt>pom.xml</tt> file location
     * @return The Document created
     * @throws ROMException If some ROM parsing problem occurs
     */
    public Document parse(final File file) throws ROMException {
        String content;
        try {
            content = FileUtils.readFileToString(file);
        } catch (java.io.IOException ex) {
            throw new ROMException(ex);
        }
        Logger.debug(
            this,
            "#parse('%s'): %d bytes file",
            file,
            content.length()
        );
        return this.parse(content);
    }

    /**
     * Parse the content provided.
     * @param rom Content of <tt>pom.xml</tt>
     * @return The Document created
     * @throws ROMException If some ROM parsing problem occurs
     */
    public Document parse(final String rom) throws ROMException {
        ROMUtils.dbFactory.setNamespaceAware(true);
        DocumentBuilder parser;
        try {
            parser = ROMUtils.dbFactory.newDocumentBuilder();
        } catch (javax.xml.parsers.ParserConfigurationException ex) {
            throw RuntimeProblem.make(ex);
        }
        Document document;
        try {
            document = parser.parse(new InputSource(new StringReader(rom)));
        } catch (org.xml.sax.SAXException ex) {
            throw new ROMException(ex);
        } catch (java.io.IOException ex) {
            throw new ROMException(ex);
        }
        new ROMValidator().validate(document);
        Logger.debug(
            this,
            "#parse(%d bytes): XML loaded",
            rom.length()
        );
        return document;
    }

    /**
     * Execute xpath on the document and return string value.
     * @param query The xpath
     * @param item What to work with
     * @param qname What to return
     * @return The result
     */
    public Object xpath(final String query, final Object item,
        final QName qname) {
        final XPath xpath = ROMUtils.xpathFactory.newXPath();
        try {
            return xpath.evaluate(
                query,
                item,
                qname
            );
        } catch (javax.xml.xpath.XPathExpressionException ex) {
            throw RuntimeProblem.make(ex);
        }
    }

    /**
     * Get one simple string from XML.
     * @param xpath The XPath
     * @param item The node to work with (the root)
     * @return The value of text()
     */
    public String text(final String xpath, final Object item) {
        final Node node = (Node) this.xpath(xpath, item, XPathConstants.NODE);
        if (node == null) {
            throw RuntimeProblem.make(xpath + " found nothing");
        }
        return node.getNodeValue();
    }

    /**
     * Convert {@link Document} to {@link String}.
     * @param doc The document to transform
     * @return The string as a result
     */
    public String transform(final Document doc) {
        try {
            final Transformer trans = this.transFactory.newTransformer();
            trans.setOutputProperty(OutputKeys.INDENT, "yes");
            final StringWriter writer = new StringWriter();
            trans.transform(new DOMSource(doc), new StreamResult(writer));
            return writer.toString();
        } catch (javax.xml.transform.TransformerConfigurationException ex) {
            throw RuntimeProblem.make(ex);
        } catch (javax.xml.transform.TransformerException ex) {
            throw RuntimeProblem.make(ex);
        }
    }

}
