/**
 * Copyright (c) 2011, REMPL.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution.
 * 3) Neither the name of the REMPL.com nor the names of its
 * contributors may be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.rempl.api.uml;

// REMPL API from com.rempl:rempl-api
import com.rempl.api.Model;

// for manipulation with slots
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * One instance at the diagram, actually an instance of
 * UML Element. An {@link Instance} is created only by the
 * {@link Diagram} class and shouldn't be instantiated outside
 * of it.
 *
 * @author Yegor Bugayenko (yegor@rempl.com)
 * @version $Id: Instance.java 1721 2011-07-27 23:58:18Z guard $
 * @see Diagram#add(String)
 */
public final class Instance {

    /**
     * UML type.
     * @see #Instance(String)
     */
    private final String type;

    /**
     * UML attributes of the instance.
     */
    private final Map<String, PrimitiveType> attributes =
        new HashMap<String, PrimitiveType>();

    /**
     * UML associations of the instance.
     */
    private final Map<String, List<Instance>> associations =
        new HashMap<String, List<Instance>>();

    /**
     * Protected ctor. It can be called only by the {@link Diagram}
     * class.
     * @param name The name of the UML type
     * @see Diagram#add(String)
     */
    protected Instance(final String name) {
        this.type = name;
    }

    /**
     * Add new attribute.
     * @param name The name of the attribute
     * @param value The value of it
     * @return This
     */
    public Instance add(final String name, final PrimitiveType value) {
        if (this.attributes.containsKey(name)) {
            throw new IllegalArgumentException(
                "Duplicate attribute \"" + name + "\""
            );
        }
        this.attributes.put(name, value);
        return this;
    }

    /**
     * Add new attribute.
     * @param name The name of the attribute
     * @param value The value of it
     * @return This
     */
    public Instance add(final String name, final String value) {
        this.add(name, new UmlString(value));
        return this;
    }

    /**
     * Add new attribute.
     * @param name The name of the attribute
     * @param value The value of it
     * @return This
     */
    public Instance add(final String name, final Boolean value) {
        this.add(name, new UmlBoolean(value));
        return this;
    }

    /**
     * Add new attribute.
     * @param name The name of the attribute
     * @param value The value of it
     * @return This
     */
    public Instance add(final String name, final Integer value) {
        this.add(name, new UmlInteger(value));
        return this;
    }

    /**
     * Add new association.
     * @param name The name of the association end
     * @param value The destination of it
     * @return This
     */
    public Instance add(final String name, final Instance value) {
        if (!this.associations.containsKey(name)) {
            this.associations.put(name, new ArrayList<Instance>());
        }
        this.associations.get(name).add(value);
        return this;
    }

    /**
     * Convert the element to UML.
     * @param diagram The diagram
     * @return The UML
     * @see Diagram#uml()
     */
    public String uml(final Diagram diagram) {
        final StringBuilder slots = new StringBuilder();
        for (String attr : this.attributes.keySet()) {
            slots.append(attr).append(":")
                .append(this.attributes.get(attr).uml()).append(";");
        }
        for (String assoc : this.associations.keySet()) {
            for (Instance val : this.associations.get(assoc)) {
                slots.append(assoc).append(":\"").append(Model.REF_PREFIX)
                    .append(diagram.instanceId(val)).append("\";");
            }
        }
        return String.format(
            "%s:%s{%s};",
            diagram.instanceId(this),
            this.type,
            slots.toString()
        );
    }

}
