/**
 * @version $Id: ROMConfigurationTest.java 1614 2011-07-10 19:05:43Z guard $
 */
package com.rempl.api;

import com.ymock.util.Logger;

import com.rempl.tk.Reflector;
import com.rempl.tk.ResourceLoader;
import com.ymock.util.RuntimeProblem;
import org.junit.Test;
import org.w3c.dom.Document;
import org.xml.sax.InputSource;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import java.io.File;
import java.io.FileInputStream;
import java.io.StringReader;
import java.util.Collection;

import static org.junit.Assert.assertEquals;

public class ROMConfigurationTest {

    @Test
    public void testSurfaceArea() throws Exception {
        Reflector.start()
            .classExists("com.rempl.api.ROMConfiguration")
                .methodExists("append(com.rempl.api.ROMConfiguration)")
                    .back() // to interface
                .methodExists("get(String, Class<?>)")
                    .back() // to interface
                .methodExists("set(String, Object)")
                    .back() // to interface
                    ;
    }

    @Test
    public void testGettingElements() throws Exception {
        ROMConfiguration config = this.makeConfig("abc");
        assertEquals("xyz", config.get("/*/abc/text()"));
    }

    @Test
    public void testGettingArrayOfStrings() throws Exception {
        ROMConfiguration config = this.makeConfig("abc");
        config.set("foo/bar1/node", "one");
        config.set("foo/bar2/node", "two");
        Collection<String> list =
            (Collection<String>) config.get("/*/foo/*/node/text()", String[].class);
        assertEquals(2, list.size());
    }

    @Test
    public void testSettingElements() throws Exception {
        ROMConfiguration config = this.makeConfig("abc");
        config.set("cde/sibling", new String("works"));
        assertEquals("works", config.get("/*/cde/sibling/text()"));
    }

    @Test
    public void testAppendingConfigurationToConfiguration() throws Exception {
        ROMConfiguration config1 = this.makeConfig("abc");
        ROMConfiguration config2 = this.makeConfig("cde");
        config2.set("foo/bar", "works");
        config1.append(config2);
        assertEquals("xyz", config1.get("/*/cde/text()"));
        assertEquals("works", config1.get("/*/foo/bar/text()"));
        Logger.debug(this, "config:\n%s", config1.toString());
    }

    @Test
    public void testAppendingEmptyToNonEmpty() throws Exception {
        ROMConfiguration config1 = this.parseConfig("abc");
        ROMConfiguration config2 = new ROMConfiguration(null);
        config1.append(config2);
        assertEquals("xyz", config1.get("/*/abc/text()"));
    }

    @Test
    public void testAppendingNonEmptyToEmpty() throws Exception {
        ROMConfiguration config1 = new ROMConfiguration(null);
        ROMConfiguration config2 = this.parseConfig("abc");
        config2.set("foo/bar/test1/test2", "works");
        config1.append(config2);
        assertEquals("xyz", config1.get("/*/abc/text()"));
        assertEquals("works", config1.get("/*/foo/bar/test1/test2/text()"));
    }

    @Test
    public void testAppendingNonEmptyToNonEmpty() throws Exception {
        ROMConfiguration config1 = new ROMConfiguration(null);
        ROMConfiguration config2 = this.parseConfig("abc");
        config1.set("foo/bar/test1", "one");
        config2.set("foo/bar/test2", "two");
        config1.append(config2);
        assertEquals("xyz", config1.get("/*/abc/text()"));
        assertEquals("one", config1.get("/*/foo/bar/test1/text()"));
        assertEquals("two", config1.get("/*/foo/bar/test2/text()"));
    }

    @Test
    public void testAppendTwoComplexFiles() throws Exception {
        ROMConfiguration config1 = this.loadConfig(ResourceLoader.find("origin.xml"));
        ROMConfiguration config2 = this.loadConfig(ResourceLoader.find("appendix-1.xml"));
        // just to make sure it doesn't fail
        config1.append(config2);
    }

    private ROMConfiguration makeConfig(String rootName) throws Exception {
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        DocumentBuilder builder = factory.newDocumentBuilder();
        Document doc = builder.newDocument();
        org.w3c.dom.Element root = doc.createElement("test");
        doc.appendChild(root);
        org.w3c.dom.Element configuration = doc.createElement("config");
        root.appendChild(configuration);
        org.w3c.dom.Element abc = doc.createElement(rootName);
        configuration.appendChild(abc);
        abc.appendChild(doc.createTextNode("xyz"));
        ROMConfiguration config = new ROMConfiguration(configuration);
        return config;
    }

    private ROMConfiguration parseConfig(String rootName) throws Exception {
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        DocumentBuilder builder = factory.newDocumentBuilder();
        String xml = String.format(
            "<root> <%s>xyz</%s> </root>",
            rootName,
            rootName
        );
        Document doc = builder.parse(new InputSource(new StringReader(xml)));
        return new ROMConfiguration(doc.getDocumentElement());
    }

    private ROMConfiguration loadConfig(File file) throws Exception {
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        DocumentBuilder builder = factory.newDocumentBuilder();
        Document doc = builder.parse(new FileInputStream(file));
        return new ROMConfiguration(doc.getDocumentElement());
    }

    @Test(expected = RuntimeProblem.class)
    public void testGettingNonStringType() throws Exception {
        ROMConfiguration config = this.makeConfig("abc");
        config.get("/*/abc/text()", Integer.class);
    }

    @Test(expected = RuntimeProblem.class)
    public void testSettingNonStringType() throws Exception {
        ROMConfiguration config = this.makeConfig("abc");
        config.set("abc/cde/foo", new Integer(123));
    }

}
